@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -55,23 +55,25 @@ import org.osgi.framework.ServiceReference;
 import org.osgi.util.tracker.ServiceTrackerCustomizer;
 import com.openexchange.conversion.DataHandler;
 import com.openexchange.conversion.DataSource;
+import com.openexchange.conversion.engine.internal.ConversionEngineRegistry;
 
 /**
  * {@link ConversionEngineCustomizer} - The service tracker customizer for conversion engine.
- * 
+ *
  * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
  */
 public final class ConversionEngineCustomizer implements ServiceTrackerCustomizer {
 
     private static final String PROP_IDENTIFIER = "identifier";
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(ConversionEngineCustomizer.class);
+    private static final org.apache.commons.logging.Log LOG =
+        com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(ConversionEngineCustomizer.class));
 
     private final BundleContext context;
 
     /**
      * Initializes a new {@link ConversionEngineCustomizer}
-     * 
+     *
      * @param context The bundle context
      */
     public ConversionEngineCustomizer(final BundleContext context) {
@@ -85,33 +87,45 @@ public final class ConversionEngineCustomizer implements ServiceTrackerCustomize
             LOG.warn("Added service is null!", new Throwable());
         }
         if (addedService instanceof DataHandler) {
-            final Object identifier = reference.getProperty(PROP_IDENTIFIER);
+            final String identifier = (String) reference.getProperty(PROP_IDENTIFIER);
             if (null == identifier) {
                 LOG.error("Missing identifier in data handler: " + addedService.getClass().getName());
                 return addedService;
             }
-            if (getInstance().getDataHandler(identifier.toString()) != null) {
-                LOG.error("A data handler is already registered for identifier: " + identifier.toString());
-                return addedService;
+            final ConversionEngineRegistry registry = getInstance();
+            synchronized (registry) {
+                if (registry.getDataHandler(identifier) != null) {
+                    LOG.error("A data handler is already registered for identifier: " + identifier);
+                    return addedService;
+                }
+                registry.putDataHandler(identifier, (DataHandler) addedService);
+                LOG.info(new StringBuilder(64).append("Data handler for identifier '").append(identifier).append(
+                    "' successfully registered"));
             }
-            getInstance().putDataHandler(identifier.toString(), (DataHandler) addedService);
-            LOG.info(new StringBuilder(64).append("Data handler for identifier '").append(identifier.toString()).append(
-                "' successfully registered"));
-        } else if (addedService instanceof DataSource) {
-            final Object identifier = reference.getProperty(PROP_IDENTIFIER);
+            return addedService;
+        }
+        if (addedService instanceof DataSource) {
+            final String identifier = (String) reference.getProperty(PROP_IDENTIFIER);
             if (null == identifier) {
                 LOG.error("Missing identifier in data source: " + addedService.getClass().getName());
                 return addedService;
             }
-            if (getInstance().getDataSource(identifier.toString()) != null) {
-                LOG.error("A data source is already registered for identifier: " + identifier.toString());
-                return addedService;
+            final ConversionEngineRegistry registry = getInstance();
+            synchronized (registry) {
+                if (registry.getDataSource(identifier) != null) {
+                    LOG.error("A data source is already registered for identifier: " + identifier);
+                    return addedService;
+                }
+                registry.putDataSource(identifier, (DataSource) addedService);
+                LOG.info(new StringBuilder(64).append("Data source for identifier '").append(identifier).append("' successfully registered"));
             }
-            getInstance().putDataSource(identifier.toString(), (DataSource) addedService);
-            LOG.info(new StringBuilder(64).append("Data source for identifier '").append(identifier.toString()).append(
-                "' successfully registered"));
+            return addedService;
         }
-        return addedService;
+        /*
+         * An untracked service?
+         */
+        context.ungetService(reference);
+        return null;
     }
 
     public void modifiedService(final ServiceReference reference, final Object service) {
@@ -121,22 +135,22 @@ public final class ConversionEngineCustomizer implements ServiceTrackerCustomize
     public void removedService(final ServiceReference reference, final Object service) {
         try {
             if (service instanceof DataHandler) {
-                final Object identifier = reference.getProperty(PROP_IDENTIFIER);
+                final String identifier = (String) reference.getProperty(PROP_IDENTIFIER);
                 if (null == identifier) {
                     LOG.error("Missing identifier in data handler: " + service.getClass().getName());
                     return;
                 }
-                getInstance().removeDataHandler(identifier.toString());
-                LOG.info(new StringBuilder(64).append("Data handler for identifier '").append(identifier.toString()).append(
+                getInstance().removeDataHandler(identifier);
+                LOG.info(new StringBuilder(64).append("Data handler for identifier '").append(identifier).append(
                     "' successfully unregistered"));
             } else if (service instanceof DataSource) {
-                final Object identifier = reference.getProperty(PROP_IDENTIFIER);
+                final String identifier = (String) reference.getProperty(PROP_IDENTIFIER);
                 if (null == identifier) {
                     LOG.error("Missing identifier in data source: " + service.getClass().getName());
                     return;
                 }
-                getInstance().removeDataSource(identifier.toString());
-                LOG.info(new StringBuilder(64).append("Data source for identifier '").append(identifier.toString()).append(
+                getInstance().removeDataSource(identifier);
+                LOG.info(new StringBuilder(64).append("Data source for identifier '").append(identifier).append(
                     "' successfully unregistered"));
             }
         } finally {
